/*
 * Copyright © 2018 Apple Inc. and the ServiceTalk project authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.servicetalk.http.router.predicate;

import java.util.Iterator;
import java.util.function.Predicate;

final class AnyMatchPredicate<V> implements Predicate<Iterator<? extends V>> {

    private final V expectedValue;

    AnyMatchPredicate(final V expectedValue) {
        this.expectedValue = expectedValue;
    }

    @Override
    public boolean test(final Iterator<? extends V> values) {
        while (values.hasNext()) {
            final V value = values.next();
            if (expectedValue.equals(value)) {
                return true;
            }
        }
        return false;
    }
}
